@@ -95,7 +95,7 @@ process_post_login(struct vsf_session* p_sess)
   }
   if (tunable_async_abor_enable)
   {
-    vsf_sysutil_install_sighandler(kVSFSysUtilSigURG, handle_sigurg, p_sess);
+    vsf_sysutil_install_sighandler(kVSFSysUtilSigURG, handle_sigurg, p_sess, 0);
     vsf_sysutil_activate_sigurg(VSFTP_COMMAND_FD);
   }
   /* Handle any login message */
@@ -125,7 +125,7 @@ process_post_login(struct vsf_session* p_sess)
       vsf_sysutil_setproctitle_str(&proctitle_str);
       str_free(&proctitle_str);
     }
-    /* Test command against the allowed list.. */
+    /* Test command against the allowed lists.. */
     if (tunable_cmds_allowed)
     {
       static struct mystr s_src_str;
@@ -146,14 +146,33 @@ process_post_login(struct vsf_session* p_sess)
         str_copy(&s_src_str, &s_rhs_str);
       }
     }
+    if (tunable_cmds_denied)
+    {
+      static struct mystr s_src_str;
+      static struct mystr s_rhs_str;
+      str_alloc_text(&s_src_str, tunable_cmds_denied);
+      while (1)
+      {
+        str_split_char(&s_src_str, &s_rhs_str, ',');
+        if (str_isempty(&s_src_str))
+        {
+          break;
+        }
+        else if (str_equal(&s_src_str, &p_sess->ftp_cmd_str))
+        {
+          cmd_ok = 0;
+          break;
+        }
+        str_copy(&s_src_str, &s_rhs_str);
+      }
+    }
     if (!cmd_ok)
     {
       vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
     }
     else if (str_equal_text(&p_sess->ftp_cmd_str, "QUIT"))
     {
-      vsf_cmdio_write(p_sess, FTP_GOODBYE, "Goodbye.");
-      vsf_sysutil_exit(0);
+      vsf_cmdio_write_exit(p_sess, FTP_GOODBYE, "Goodbye.");
     }
     else if (str_equal_text(&p_sess->ftp_cmd_str, "PWD") ||
              str_equal_text(&p_sess->ftp_cmd_str, "XPWD"))
@@ -317,7 +336,9 @@ process_post_login(struct vsf_session* p_sess)
         vsf_cmdio_write(p_sess, FTP_BADMODE, "Bad MODE command.");
       }
     }
-    else if (str_equal_text(&p_sess->ftp_cmd_str, "STOU"))
+    else if (tunable_write_enable &&
+             (tunable_anon_upload_enable || !p_sess->is_anonymous) &&
+             str_equal_text(&p_sess->ftp_cmd_str, "STOU"))
     {
       handle_stou(p_sess);
     }
@@ -401,6 +422,11 @@ process_post_login(struct vsf_session* p_sess)
     {
       vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
     }
+    else if (str_isempty(&p_sess->ftp_cmd_str) &&
+             str_isempty(&p_sess->ftp_arg_str))
+    {
+      /* Deliberately ignore to avoid NAT device bugs. ProFTPd does the same. */
+    }
     else
     {
       vsf_cmdio_write(p_sess, FTP_BADCMD, "Unknown command.");
@@ -476,9 +502,16 @@ static int
 pasv_active(struct vsf_session* p_sess)
 {
   int ret = 0;
-  if (p_sess->pasv_listen_fd != -1)
+  if (tunable_one_process_model)
+  {
+    ret = vsf_one_process_pasv_active(p_sess);
+  }
+  else
+  {
+    ret = vsf_two_process_pasv_active(p_sess);
+  }
+  if (ret)
   {
-    ret = 1;
     if (port_active(p_sess))
     {
       bug("pasv and port both active");
@@ -496,23 +529,22 @@ port_cleanup(struct vsf_session* p_sess)
 static void
 pasv_cleanup(struct vsf_session* p_sess)
 {
-  if (p_sess->pasv_listen_fd != -1)
+  if (tunable_one_process_model)
   {
-    vsf_sysutil_close(p_sess->pasv_listen_fd);
-    p_sess->pasv_listen_fd = -1;
+    vsf_one_process_pasv_cleanup(p_sess);
+  }
+  else
+  {
+    vsf_two_process_pasv_cleanup(p_sess);
   }
 }
 
 static void
 handle_pasv(struct vsf_session* p_sess, int is_epsv)
 {
+  unsigned short the_port;
   static struct mystr s_pasv_res_str;
   static struct vsf_sysutil_sockaddr* s_p_sockaddr;
-  int bind_retries = 10;
-  unsigned short the_port = 0;
-  /* IPPORT_RESERVED */
-  unsigned short min_port = 1024;
-  unsigned short max_port = 65535;
   int is_ipv6 = vsf_sysutil_sockaddr_is_ipv6(p_sess->p_local_addr);
   if (is_epsv && !str_isempty(&p_sess->ftp_arg_str))
   {
@@ -533,59 +565,19 @@ handle_pasv(struct vsf_session* p_sess, int is_epsv)
   }
   pasv_cleanup(p_sess);
   port_cleanup(p_sess);
-  if (is_ipv6)
+  if (tunable_one_process_model)
   {
-    p_sess->pasv_listen_fd = vsf_sysutil_get_ipv6_sock();
+    the_port = vsf_one_process_listen(p_sess);
   }
   else
   {
-    p_sess->pasv_listen_fd = vsf_sysutil_get_ipv4_sock();
+    the_port = vsf_two_process_listen(p_sess);
   }
-  vsf_sysutil_activate_reuseaddr(p_sess->pasv_listen_fd);
-
-  if (tunable_pasv_min_port > min_port && tunable_pasv_min_port <= max_port)
-  {
-    min_port = tunable_pasv_min_port;
-  }
-  if (tunable_pasv_max_port >= min_port && tunable_pasv_max_port < max_port)
-  {
-    max_port = tunable_pasv_max_port;
-  }
-
-  while (--bind_retries)
-  {
-    int retval;
-    double scaled_port;
-    the_port = vsf_sysutil_get_random_byte();
-    the_port <<= 8;
-    the_port |= vsf_sysutil_get_random_byte();
-    scaled_port = (double) min_port;
-    scaled_port += ((double) the_port / (double) 65536) *
-                   ((double) max_port - min_port + 1);
-    the_port = (unsigned short) scaled_port;
-    vsf_sysutil_sockaddr_clone(&s_p_sockaddr, p_sess->p_local_addr);
-    vsf_sysutil_sockaddr_set_port(s_p_sockaddr, the_port);
-    retval = vsf_sysutil_bind(p_sess->pasv_listen_fd, s_p_sockaddr);
-    if (!vsf_sysutil_retval_is_error(retval))
-    {
-      break;
-    }
-    if (vsf_sysutil_get_error() == kVSFSysUtilErrADDRINUSE)
-    {
-      continue;
-    }
-    die("vsf_sysutil_bind");
-  }
-  if (!bind_retries)
-  {
-    die("vsf_sysutil_bind");
-  }
-  vsf_sysutil_listen(p_sess->pasv_listen_fd, 1);
   if (is_epsv)
   {
     str_alloc_text(&s_pasv_res_str, "Entering Extended Passive Mode (|||");
     str_append_ulong(&s_pasv_res_str, (unsigned long) the_port);
-    str_append_text(&s_pasv_res_str, "|)");
+    str_append_text(&s_pasv_res_str, "|).");
     vsf_cmdio_write_str(p_sess, FTP_EPSVOK, &s_pasv_res_str);
     return;
   }
@@ -597,6 +589,10 @@ handle_pasv(struct vsf_session* p_sess, int is_epsv)
       die("invalid pasv_address");
     }
   }
+  else
+  {
+    vsf_sysutil_sockaddr_clone(&s_p_sockaddr, p_sess->p_local_addr);
+  }
   str_alloc_text(&s_pasv_res_str, "Entering Passive Mode (");
   if (!is_ipv6)
   {
@@ -619,7 +615,7 @@ handle_pasv(struct vsf_session* p_sess, int is_epsv)
   str_append_ulong(&s_pasv_res_str, the_port >> 8);
   str_append_text(&s_pasv_res_str, ",");
   str_append_ulong(&s_pasv_res_str, the_port & 255);
-  str_append_text(&s_pasv_res_str, ")");
+  str_append_text(&s_pasv_res_str, ").");
   vsf_cmdio_write_str(p_sess, FTP_PASVOK, &s_pasv_res_str);
 }
 
@@ -718,7 +714,10 @@ handle_retr(struct vsf_session* p_sess)
   }
   trans_ret = vsf_ftpdataio_transfer_file(p_sess, remote_fd,
                                           opened_file, 0, is_ascii);
-  vsf_ftpdataio_dispose_transfer_fd(p_sess);
+  if (vsf_ftpdataio_dispose_transfer_fd(p_sess) != 1 && trans_ret.retval == 0)
+  {
+    trans_ret.retval = -2;
+  }
   p_sess->transfer_size = trans_ret.transferred;
   /* Log _after_ the blocking dispose call, so we get transfer times right */
   if (trans_ret.retval == 0)
@@ -863,7 +862,10 @@ handle_dir_common(struct vsf_session* p_sess, int full_details, int stat_cmd)
   }
   if (!stat_cmd)
   {
-    vsf_ftpdataio_dispose_transfer_fd(p_sess);
+    if (vsf_ftpdataio_dispose_transfer_fd(p_sess) != 1 && retval == 0)
+    {
+      retval = -1;
+    }
   }
   if (stat_cmd)
   {
@@ -972,6 +974,9 @@ handle_upload_common(struct vsf_session* p_sess, int is_append, int is_unique)
   struct vsf_transfer_ret trans_ret;
   int new_file_fd;
   int remote_fd;
+  int success = 0;
+  int created = 0;
+  int do_truncate = 0;
   filesize_t offset = p_sess->restart_pos;
   p_sess->restart_pos = 0;
   if (!data_transfer_checks_ok(p_sess))
@@ -1004,13 +1009,10 @@ handle_upload_common(struct vsf_session* p_sess, int is_append, int is_unique)
   else
   {
     /* For non-anonymous, allow open() to overwrite or append existing files */
+    new_file_fd = str_create_append(p_filename);
     if (!is_append && offset == 0)
     {
-      new_file_fd = str_create_overwrite(p_filename);
-    }
-    else
-    {
-      new_file_fd = str_create_append(p_filename);
+      do_truncate = 1;
     }
   }
   if (vsf_sysutil_retval_is_error(new_file_fd))
@@ -1018,6 +1020,7 @@ handle_upload_common(struct vsf_session* p_sess, int is_append, int is_unique)
     vsf_cmdio_write(p_sess, FTP_UPLOADFAIL, "Could not create file.");
     return;
   }
+  created = 1;
   vsf_sysutil_fstat(new_file_fd, &s_p_statbuf);
   if (vsf_sysutil_statbuf_is_regfile(s_p_statbuf))
   {
@@ -1044,9 +1047,19 @@ handle_upload_common(struct vsf_session* p_sess, int is_append, int is_unique)
   {
     vsf_sysutil_lock_file_write(new_file_fd);
   }
+  /* Must truncate the file AFTER locking it! */
+  if (do_truncate)
+  {
+    vsf_sysutil_ftruncate(new_file_fd);
+    vsf_sysutil_lseek_to(new_file_fd, 0);
+  }
   if (!is_append && offset != 0)
   {
     /* XXX - warning, allows seek past end of file! Check for seek > size? */
+    /* XXX - also, currently broken as the O_APPEND flag will always write
+     * at the end of file. No known complaints yet; can easily fix if one
+     * comes in.
+     */
     vsf_sysutil_lseek_to(new_file_fd, offset);
   }
   if (is_unique)
@@ -1075,18 +1088,21 @@ handle_upload_common(struct vsf_session* p_sess, int is_append, int is_unique)
     trans_ret = vsf_ftpdataio_transfer_file(p_sess, remote_fd,
                                             new_file_fd, 1, 0);
   }
-  vsf_ftpdataio_dispose_transfer_fd(p_sess);
+  if (vsf_ftpdataio_dispose_transfer_fd(p_sess) != 1 && trans_ret.retval == 0)
+  {
+    trans_ret.retval = -2;
+  }
   p_sess->transfer_size = trans_ret.transferred;
-  /* XXX - handle failure, delete file? */
   if (trans_ret.retval == 0)
   {
+    success = 1;
     vsf_log_do_log(p_sess, 1);
   }
   if (trans_ret.retval == -1)
   {
     vsf_cmdio_write(p_sess, FTP_BADSENDFILE, "Failure writing to local file.");
   }
-  else if (trans_ret.retval == -2)
+  else if (trans_ret.retval == -2 || p_sess->abor_received)
   {
     vsf_cmdio_write(p_sess, FTP_BADSENDNET, "Failure reading network stream.");
   }
@@ -1098,6 +1114,10 @@ handle_upload_common(struct vsf_session* p_sess, int is_append, int is_unique)
 port_pasv_cleanup_out:
   port_cleanup(p_sess);
   pasv_cleanup(p_sess);
+  if (tunable_delete_failed_uploads && created && !success)
+  {
+    str_unlink(p_filename);
+  }
   vsf_sysutil_close(new_file_fd);
 }
 
@@ -1600,6 +1620,7 @@ handle_eprt(struct vsf_session* p_sess)
 {
   static struct mystr s_part1_str;
   static struct mystr s_part2_str;
+  static struct mystr s_scopeid_str;
   int proto;
   int port;
   const unsigned char* p_raw_addr;
@@ -1624,6 +1645,7 @@ handle_eprt(struct vsf_session* p_sess)
   str_split_char(&s_part1_str, &s_part2_str, '|');
   if (proto == 2)
   {
+    str_split_char(&s_part1_str, &s_scopeid_str, '%');
     p_raw_addr = vsf_sysutil_parse_ipv6(&s_part1_str);
   }
   else
@@ -1708,7 +1730,13 @@ get_unique_filename(struct mystr* p_outstr, const struct mystr* p_base_str)
    */
   static struct vsf_sysutil_statbuf* s_p_statbuf;
   unsigned int suffix = 1;
-  int retval;
+  /* Do not add any suffix at all if the name is not taken. */
+  int retval = str_stat(p_base_str, &s_p_statbuf);
+  if (vsf_sysutil_retval_is_error(retval))
+  {
+    str_copy(p_outstr, p_base_str);
+    return;
+  }
   while (1)
   {
     str_copy(p_outstr, p_base_str);
