@@ -127,8 +127,7 @@ parse_username_password(struct vsf_session* p_sess)
     }
     else if (str_equal_text(&p_sess->ftp_cmd_str, "QUIT"))
     {
-      vsf_cmdio_write(p_sess, FTP_GOODBYE, "Goodbye.");
-      vsf_sysutil_exit(0);
+      vsf_cmdio_write_exit(p_sess, FTP_GOODBYE, "Goodbye.");
     }
     else if (str_equal_text(&p_sess->ftp_cmd_str, "FEAT"))
     {
@@ -155,7 +154,7 @@ parse_username_password(struct vsf_session* p_sess)
     else if (str_isempty(&p_sess->ftp_cmd_str) &&
              str_isempty(&p_sess->ftp_arg_str))
     {
-      // Deliberately ignore to avoid NAT device bugs. ProFTPd does the same.
+      /* Deliberately ignore to avoid NAT device bugs. ProFTPd does the same. */
     }
     else
     {
@@ -217,6 +216,7 @@ handle_user_command(struct vsf_session* p_sess)
     if ((located && tunable_userlist_deny) ||
         (!located && !tunable_userlist_deny))
     {
+      check_login_delay();
       vsf_cmdio_write(p_sess, FTP_LOGINERR, "Permission denied.");
       check_login_fails(p_sess);
       str_empty(&p_sess->user_str);
