@@ -85,6 +85,8 @@ ftp_read_data(struct vsf_session* p_sess, char* p_buf, unsigned int len)
     priv_sock_send_cmd(p_sess->ssl_consumer_fd, PRIV_SOCK_DO_SSL_READ);
     ret = priv_sock_get_int(p_sess->ssl_consumer_fd);
     priv_sock_recv_buf(p_sess->ssl_consumer_fd, p_buf, len);
+    /* Need to do this here too because it is useless in the slave process. */
+    vsf_sysutil_check_pending_actions(kVSFSysUtilIO, ret, p_sess->data_fd);
     return ret;
   }
   else if (p_sess->data_use_ssl)
@@ -103,9 +105,13 @@ ftp_write_data(const struct vsf_session* p_sess, const char* p_buf,
 {
   if (p_sess->data_use_ssl && p_sess->ssl_slave_active)
   {
+    int ret;
     priv_sock_send_cmd(p_sess->ssl_consumer_fd, PRIV_SOCK_DO_SSL_WRITE);
     priv_sock_send_buf(p_sess->ssl_consumer_fd, p_buf, len);
-    return priv_sock_get_int(p_sess->ssl_consumer_fd);
+    ret = priv_sock_get_int(p_sess->ssl_consumer_fd);
+    /* Need to do this here too because it is useless in the slave process. */
+    vsf_sysutil_check_pending_actions(kVSFSysUtilIO, ret, p_sess->data_fd);
+    return ret;
   }
   else if (p_sess->data_use_ssl)
   {
