@@ -15,6 +15,7 @@
 #include "sysutil.h"
 #include "utility.h"
 #include "tunables.h"
+#include "sysdeputil.h"
 
 /* Activate 64-bit file support on Linux/32bit plus others */
 #define _FILE_OFFSET_BITS 64
@@ -116,8 +117,9 @@ vsf_sysutil_common_sighandler(int signum)
 {
   if (signum < 0 || signum >= NSIG)
   {
-    // bug() is not async safe but this check really is a "cannot happen"
-    // debug aid.
+    /* bug() is not async safe but this check really is a "cannot happen"
+     * debug aid.
+     */
     bug("signal out of range in vsf_sysutil_common_sighandler");
   }
   if (s_sig_details[signum].sync_sig_handler)
@@ -447,7 +449,7 @@ vsf_sysutil_write_loop(const int fd, const void* p_buf, unsigned int size)
     }
     if ((unsigned int) retval > size)
     {
-      die("retval too big in vsf_sysutil_read_loop");
+      die("retval too big in vsf_sysutil_write_loop");
     }
     num_written += retval;
     size -= (unsigned int) retval;
@@ -533,7 +535,7 @@ vsf_sysutil_getpid(void)
 {
   if (s_current_pid == -1)
   {
-    s_current_pid = getpid();
+    s_current_pid = vsf_sysutil_getpid_nocache();
   }
   return (unsigned int) s_current_pid;
 }
@@ -541,15 +543,7 @@ vsf_sysutil_getpid(void)
 int
 vsf_sysutil_fork(void)
 {
-  /* Child does NOT inherit exit function */
-  exitfunc_t curr_func = s_exit_func;
-  int retval;
-  s_exit_func = 0;
-  retval = vsf_sysutil_fork_failok();
-  if (retval != 0)
-  {
-    s_exit_func = curr_func;
-  }
+  int retval = vsf_sysutil_fork_failok();
   if (retval < 0)
   {
     die("fork");
@@ -560,10 +554,18 @@ vsf_sysutil_fork(void)
 int
 vsf_sysutil_fork_failok(void)
 {
-  int retval = fork();
+  /* Child does NOT inherit exit function */
+  exitfunc_t curr_func = s_exit_func;
+  int retval;
+  s_exit_func = 0;
+  retval = fork();
+  if (retval != 0)
+  {
+    s_exit_func = curr_func;
+  }
   if (retval == 0)
   {
-    s_current_pid = -1;
+    vsf_sysutil_clear_pid_cache();
   }
   return retval;
 }
@@ -687,7 +689,7 @@ vsf_sysutil_set_nodelay(int fd)
 void
 vsf_sysutil_activate_sigurg(int fd)
 {
-  int retval = fcntl(fd, F_SETOWN, getpid());
+  int retval = fcntl(fd, F_SETOWN, vsf_sysutil_getpid());
   if (retval != 0)
   {
     die("fcntl");
@@ -1596,7 +1598,10 @@ vsf_sysutil_get_error(void)
       retval = kVSFSysUtilErrOPNOTSUPP;
       break;
     case EACCES:
-      retval = kVSFSysUtilErrOPNOTSUPP;
+      retval = kVSFSysUtilErrACCES;
+      break;
+    case ENOENT:
+      retval = kVSFSysUtilErrNOENT;
       break;
   }
   return retval;
@@ -1683,7 +1688,7 @@ vsf_sysutil_accept_timeout(int fd, struct vsf_sysutil_sockaddr* p_sockaddr,
   int saved_errno;
   fd_set accept_fdset;
   struct timeval timeout;
-  unsigned int socklen = sizeof(remote_addr);
+  socklen_t socklen = sizeof(remote_addr);
   if (p_sockaddr)
   {
     vsf_sysutil_memclr(p_sockaddr, sizeof(*p_sockaddr));
@@ -1807,7 +1812,7 @@ vsf_sysutil_getsockname(int fd, struct vsf_sysutil_sockaddr** p_sockptr)
 {
   struct vsf_sysutil_sockaddr the_addr;
   int retval;
-  unsigned int socklen = sizeof(the_addr);
+  socklen_t socklen = sizeof(the_addr);
   vsf_sysutil_sockaddr_clear(p_sockptr);
   retval = getsockname(fd, &the_addr.u.u_sockaddr, &socklen);
   if (retval != 0)
@@ -1832,7 +1837,7 @@ vsf_sysutil_getpeername(int fd, struct vsf_sysutil_sockaddr** p_sockptr)
 {
   struct vsf_sysutil_sockaddr the_addr;
   int retval;
-  unsigned int socklen = sizeof(the_addr);
+  socklen_t socklen = sizeof(the_addr);
   vsf_sysutil_sockaddr_clear(p_sockptr);
   retval = getpeername(fd, &the_addr.u.u_sockaddr, &socklen);
   if (retval != 0)
@@ -1924,6 +1929,8 @@ vsf_sysutil_sockaddr_clone(struct vsf_sysutil_sockaddr** p_sockptr,
     vsf_sysutil_memcpy(&p_sockaddr->u.u_sockaddr_in6.sin6_addr,
                        &p_src->u.u_sockaddr_in6.sin6_addr,
                        sizeof(p_sockaddr->u.u_sockaddr_in6.sin6_addr));
+    p_sockaddr->u.u_sockaddr_in6.sin6_scope_id =
+        p_src->u.u_sockaddr_in6.sin6_scope_id;
   }
   else
   {
@@ -2129,6 +2136,25 @@ vsf_sysutil_sockaddr_set_any(struct vsf_sysutil_sockaddr* p_sockaddr)
   }
 }
 
+unsigned short
+vsf_sysutil_sockaddr_get_port(const struct vsf_sysutil_sockaddr* p_sockptr)
+{
+  if (p_sockptr->u.u_sockaddr.sa_family == AF_INET)
+  {
+    return ntohs(p_sockptr->u.u_sockaddr_in.sin_port);
+  }
+  else if (p_sockptr->u.u_sockaddr.sa_family == AF_INET6)
+  {
+    return ntohs(p_sockptr->u.u_sockaddr_in6.sin6_port);
+  }
+  else
+  {
+    bug("bad family");
+  }
+  /* NOTREACHED */
+  return 0;
+}
+
 void
 vsf_sysutil_sockaddr_set_port(struct vsf_sysutil_sockaddr* p_sockptr,
                               unsigned short the_port)
@@ -2483,7 +2509,7 @@ vsf_sysutil_make_session_leader(void)
   /* This makes us the leader if we are not already */
   (void) setsid();
   /* Check we're the leader */
-  if (getpid() != getpgrp())
+  if ((int) vsf_sysutil_getpid() != getpgrp())
   {
     die("not session leader");
   }
@@ -2760,3 +2786,39 @@ vsf_sysutil_set_address_space_limit(long bytes)
 #endif /* RLIMIT_AS */
   (void) bytes;
 }
+
+void
+vsf_sysutil_set_no_fds()
+{
+  int ret;
+  struct rlimit rlim;
+  rlim.rlim_cur = 0;
+  rlim.rlim_max = 0;
+  ret = setrlimit(RLIMIT_NOFILE, &rlim);
+  if (ret != 0)
+  {
+    die("setrlimit NOFILE");
+  }
+}
+
+void
+vsf_sysutil_set_no_procs()
+{
+#ifdef RLIMIT_NPROC
+  int ret;
+  struct rlimit rlim;
+  rlim.rlim_cur = 0;
+  rlim.rlim_max = 0;
+  ret = setrlimit(RLIMIT_NPROC, &rlim);
+  if (ret != 0)
+  {
+    die("setrlimit NPROC");
+  }
+#endif
+}
+
+void
+vsf_sysutil_clear_pid_cache()
+{
+  s_current_pid = -1;
+}
