@@ -16,7 +16,9 @@ enum EVSFSysUtilError
   kVSFSysUtilErrNOSYS,
   kVSFSysUtilErrINTR,
   kVSFSysUtilErrINVAL,
-  kVSFSysUtilErrOPNOTSUPP
+  kVSFSysUtilErrOPNOTSUPP,
+  kVSFSysUtilErrACCES,
+  kVSFSysUtilErrNOENT
 };
 enum EVSFSysUtilError vsf_sysutil_get_error(void);
 
@@ -41,7 +43,9 @@ typedef void (*vsf_context_io_t)(int, int, void*);
 
 void vsf_sysutil_install_null_sighandler(const enum EVSFSysUtilSignal sig);
 void vsf_sysutil_install_sighandler(const enum EVSFSysUtilSignal,
-                                    vsf_sighandle_t handler, void* p_private);
+                                    vsf_sighandle_t handler,
+                                    void* p_private,
+                                    int use_alarm);
 void vsf_sysutil_install_async_sighandler(const enum EVSFSysUtilSignal sig,
                                           vsf_async_sighandle_t handler);
 void vsf_sysutil_default_sig(const enum EVSFSysUtilSignal sig);
@@ -90,6 +94,7 @@ void vsf_sysutil_close(int fd);
 int vsf_sysutil_close_failok(int fd);
 int vsf_sysutil_unlink(const char* p_dead);
 int vsf_sysutil_write_access(const char* p_filename);
+void vsf_sysutil_ftruncate(int fd);
 
 /* Reading and writing */
 void vsf_sysutil_lseek_to(const int fd, filesize_t seek_pos);
@@ -160,6 +165,7 @@ void vsf_sysutil_free(void* p_ptr);
 
 /* Process creation/exit/process handling */
 unsigned int vsf_sysutil_getpid(void);
+void vsf_sysutil_clear_pid_cache(void);
 int vsf_sysutil_fork(void);
 int vsf_sysutil_fork_failok(void);
 void vsf_sysutil_exit(int exit_code);
@@ -223,6 +229,8 @@ void vsf_sysutil_sockaddr_set_ipv4addr(struct vsf_sysutil_sockaddr* p_sockptr,
 void vsf_sysutil_sockaddr_set_ipv6addr(struct vsf_sysutil_sockaddr* p_sockptr,
                                        const unsigned char* p_raw);
 void vsf_sysutil_sockaddr_set_any(struct vsf_sysutil_sockaddr* p_sockaddr);
+unsigned short vsf_sysutil_sockaddr_get_port(
+    const struct vsf_sysutil_sockaddr* p_sockptr);
 void vsf_sysutil_sockaddr_set_port(struct vsf_sysutil_sockaddr* p_sockptr,
                                    unsigned short the_port);
 int vsf_sysutil_is_port_reserved(unsigned short port);
@@ -239,7 +247,7 @@ int vsf_sysutil_get_ipv6_sock(void);
 struct vsf_sysutil_socketpair_retval
   vsf_sysutil_unix_stream_socketpair(void);
 int vsf_sysutil_bind(int fd, const struct vsf_sysutil_sockaddr* p_sockptr);
-void vsf_sysutil_listen(int fd, const unsigned int backlog);
+int vsf_sysutil_listen(int fd, const unsigned int backlog);
 void vsf_sysutil_getsockname(int fd, struct vsf_sysutil_sockaddr** p_sockptr);
 void vsf_sysutil_getpeername(int fd, struct vsf_sysutil_sockaddr** p_sockptr);
 int vsf_sysutil_accept_timeout(int fd, struct vsf_sysutil_sockaddr* p_sockaddr,
@@ -293,6 +301,7 @@ unsigned char vsf_sysutil_get_random_byte(void);
 unsigned int vsf_sysutil_get_umask(void);
 void vsf_sysutil_set_umask(unsigned int umask);
 void vsf_sysutil_make_session_leader(void);
+void vsf_sysutil_reopen_standard_fds(void);
 void vsf_sysutil_tzset(void);
 const char* vsf_sysutil_get_current_date(void);
 void vsf_sysutil_qsort(void* p_base, unsigned int num_elem,
@@ -301,10 +310,12 @@ void vsf_sysutil_qsort(void* p_base, unsigned int num_elem,
 char* vsf_sysutil_getenv(const char* p_var);
 typedef void (*exitfunc_t)(void);
 void vsf_sysutil_set_exit_func(exitfunc_t exitfunc);
+int vsf_sysutil_getuid(void);
 
 /* Syslogging (bah) */
-void vsf_sysutil_openlog(void);
+void vsf_sysutil_openlog(int force);
 void vsf_sysutil_syslog(const char* p_text, int severe);
+void vsf_sysutil_closelog(void);
 
 /* Credentials handling */
 int vsf_sysutil_running_as_root(void);
@@ -330,5 +341,10 @@ long vsf_sysutil_parse_time(const char* p_text);
 void vsf_sysutil_sleep(double seconds);
 int vsf_sysutil_setmodtime(const char* p_file, long the_time, int is_localtime);
 
+/* Limits */
+void vsf_sysutil_set_address_space_limit(long bytes);
+void vsf_sysutil_set_no_fds(void);
+void vsf_sysutil_set_no_procs(void);
+
 #endif /* VSF_SYSUTIL_H */
 
