@@ -14,6 +14,7 @@ fi
 if find_func pam_start sysdeputil.o; then
   locate_library /lib/libpam.so.0 && echo "/lib/libpam.so.0";
   locate_library /usr/lib/libpam.so && echo "-lpam";
+  locate_library /usr/lib64/libpam.so && echo "-lpam";
   # HP-UX ends shared libraries with .sl
   locate_library /usr/lib/libpam.sl && echo "-lpam";
   # AIX ends shared libraries with .a
@@ -21,6 +22,7 @@ if find_func pam_start sysdeputil.o; then
 else
   locate_library /lib/libcrypt.so && echo "-lcrypt";
   locate_library /usr/lib/libcrypt.so && echo "-lcrypt";
+  locate_library /usr/lib64/libcrypt.so && echo "-lcrypt";
 fi
 
 # Look for the dynamic linker library. Needed by older RedHat when
@@ -45,9 +47,12 @@ locate_library /usr/lib/libsec.sl && echo "-lsec";
 # Look for libcap (capabilities)
 if locate_library /lib/libcap.so.1; then
   echo "/lib/libcap.so.1";
+elif locate_library /lib/libcap.so.2; then
+  echo "/lib/libcap.so.2";
 else
   locate_library /usr/lib/libcap.so && echo "-lcap";
   locate_library /lib/libcap.so && echo "-lcap";
+  locate_library /lib64/libcap.so && echo "-lcap";
 fi
 
 # Solaris needs this for nanosleep()..
